"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * eventBus
 */
var warning_1 = __importDefault(require("warning"));
// eslint-disable-next-line @typescript-eslint/no-unused-vars
function default_1(Vue) {
    var events = {};
    return {
        /**
         * 发射事件
         * @param {string} eventName 事件名称
         * @param {any} payload 事件载荷
         */
        emit: function (eventName, payload) {
            var e_1, _a;
            if (!events[eventName]) {
                return (0, warning_1.default)(process.env.NODE_ENV === 'production', "[moduleLoader] event \"".concat(eventName, "\" is triggered, but no listeners\uFF0Cnothing will be happening."));
            }
            try {
                for (var _b = __values(events[eventName].values()), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var func = _c.value;
                    func(payload);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        },
        /**
         * 事件监听
         * @param eventName 事件名称
         * @param handler 处理事件的方法
         */
        on: function (eventName, handler) {
            if (typeof handler === 'function') {
                if (!events[eventName])
                    events[eventName] = new Set();
                events[eventName].add(handler);
            }
            else {
                (0, warning_1.default)(false, "[moduleLoader] handler must be a function, but recived ".concat(typeof handler));
            }
        },
        /**
         * 取消事件监听
         * @param eventName 事件名称
         * @param handler 处理事件的方法
         */
        off: function (eventName, handler) {
            if (!handler) {
                return (0, warning_1.default)(false, '[moduleLoader] handler must be a valid event function');
            }
            else if (!events.hasOwnProperty(eventName)) {
                return (0, warning_1.default)(process.env.NODE_ENV === 'production', "[moduleLoader] no event name of \"".concat(eventName, "\" is on listening"));
            }
            events[eventName].delete(handler);
        },
        /**
         * 清理事件总线
         */
        clear: function () {
            events = {};
        },
        /**
         * 获取当前事件总线详情
         */
        getEvents: function () {
            return events;
        },
    };
}
exports.default = default_1;
//# sourceMappingURL=eventBus.js.map